"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const admin_client_1 = require("./admin-client");
const hatchet_client_test_1 = require("../hatchet-client/hatchet-client.test");
describe('AdminClient', () => {
    let client;
    it('should create a client', () => {
        const x = new admin_client_1.AdminClient({
            token: 'TOKEN',
            host_port: 'HOST_PORT',
            tls_config: {
                cert_file: 'TLS_CERT_FILE',
                key_file: 'TLS_KEY_FILE',
                ca_file: 'TLS_ROOT_CA_FILE',
                server_name: 'TLS_SERVER_NAME',
            },
        }, hatchet_client_test_1.mockChannel, hatchet_client_test_1.mockFactory);
        expect(x).toBeDefined();
    });
    beforeEach(() => {
        client = new admin_client_1.AdminClient({
            token: 'TOKEN',
            host_port: 'HOST_PORT',
            tls_config: {
                cert_file: 'TLS_CERT_FILE',
                key_file: 'TLS_KEY_FILE',
                ca_file: 'TLS_ROOT_CA_FILE',
                server_name: 'TLS_SERVER_NAME',
            },
        }, hatchet_client_test_1.mockChannel, hatchet_client_test_1.mockFactory);
    });
    describe('put_workflow', () => {
        it('should throw an error if no version and not auto version', () => __awaiter(void 0, void 0, void 0, function* () {
            const workflow = {
                name: 'workflow1',
                version: '',
                description: 'description1',
                eventTriggers: [],
                cronTriggers: [],
                scheduledTriggers: [],
                jobs: [],
            };
            expect(() => client.put_workflow(workflow, { autoVersion: false })).rejects.toThrow('PutWorkflow error: workflow version is required, or use autoVersion');
        }));
        it('should attempt to put the workflow', () => __awaiter(void 0, void 0, void 0, function* () {
            const workflow = {
                name: 'workflow1',
                version: 'v0.0.1',
                description: 'description1',
                eventTriggers: [],
                cronTriggers: [],
                scheduledTriggers: [],
                jobs: [],
            };
            const putSpy = jest.spyOn(client.client, 'putWorkflow').mockResolvedValue({
                id: 'workflow1',
                version: 'v0.1.0',
                order: 1,
                workflowId: 'workflow1',
                jobs: [],
                createdAt: undefined,
                updatedAt: undefined,
                triggers: undefined,
            });
            yield client.put_workflow(workflow);
            expect(putSpy).toHaveBeenCalled();
        }));
    });
    describe('schedule_workflow', () => {
        it('should schedule a workflow', () => {
            const res = {
                id: 'string',
                version: 'v0.0.1',
                order: 1,
                workflowId: 'string',
                jobs: [],
                createdAt: undefined,
                updatedAt: undefined,
                triggers: undefined,
            };
            const spy = jest.spyOn(client.client, 'scheduleWorkflow').mockResolvedValue(res);
            const now = new Date();
            client.schedule_workflow('workflowId', {
                schedules: [now],
            });
            expect(spy).toHaveBeenCalledWith({
                workflowId: 'workflowId',
                schedules: [now],
            });
        });
    });
});
